#! /bin/sh
#
# Copies properties files from ./jimm to ./classes. For resource bundles,
# creates the (for lack of a better phrase) "superclass" files. For
# example, if datavision_fr_FR.properties exists but datavision_fr.properties
# does not and no other datavision_fr_*.properties files exist, we copy
# datavision_fr_FR.properties to datavision_fr.properties.
#
# We also copy the default (US English) files.
#
# One more thing: we make sure the files have permission 644.
#

here=`dirname $0`
from=$here/jimm
to=$here/classes

# Copy properties files for languages with more than one region.
for s in "pt_BR" "pt_PT" ; do
    cp $from/datavision_$s.properties $to
    # Not all of them have paper size properties files.
    if [ -f $from/paper_$s.properties ] ; then
	cp $from/paper_$s.properties $to
    fi
done

# Fill in the holes by providing "superclass" properties files. For example,
# copy "datavision_bg_BG.properties" to "$to/datavision_bg.properties".
# For languages with more than one region (e.g., pt_PT and pt_BR), we
# pick a "dominant" version.
for s in "bg_BG" "es_MX" "fr_FR" "it_IT" "pt_PT" "ru_RU" ; do
    cp $from/datavision_$s.properties $to/datavision_${s%_*}.properties
    cp $from/paper_$s.properties $to/paper_${s%_*}.properties
done

# Copy the default (US English) files.
cp $from/datavision.properties $from/paper.properties $to

# Change permissions.
chmod 644 $to/*.properties
