package jimm.datavision;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Locale;
import java.util.MissingResourceException;

/**
 * This class finds the local version of any string. It also contains
 * a method for changing the language.
 * <p>
 * Each language should have a file called datavision_XX_YY.properties,
 * where XX is the language code (e.g., "en" for English, "fr" for French)
 * and YY is the country code (e.g., "US", "FR").
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class I18N {

protected static final String RESOURCE_FILE_PREFIX = "datavision";

protected static Locale locale;
protected static ResourceBundle strings;

// Initialize the language.
static {
    setLanguage(Locale.getDefault());
}

/**
 * Given a locale, start using the code short phrases for that lanuage.
 * Normally, you won't have to call this method. It gets called at startup
 * and sets the locale to the default one.
 *
 * @param local the new locale.
 */
public static void setLanguage(Locale l) {
    if (!l.equals(locale)) {
	locale = l;
	strings = ResourceBundle.getBundle(RESOURCE_FILE_PREFIX, locale);
    }
}

/**
 * Returns the string corresponding to the specified string
 * or <code>null</code> if there isn't one. Returns <var>key</var> if
 * <var>,ey</var> is either <code>null</code> or the empty string.
 *
 * @return the string corresponding to the specified string
 * or <code>null</code> if there isn't one; if <var>key</var> is the
 * empty string, return it
 */
public static String get(String key) {
    if (key == null) return null;
    if (key.length() == 0) return "";

    String val = null;
    try {
	val = strings.getString(key);
    }
    catch (MissingResourceException ex) {
	ErrorHandler.error(ex);
    }
    return val;
}

}
