package jimm.datavision;
import java.io.*;
//  import java.util.*;
import uk.co.wilson.xml.MinML;
import org.xml.sax.*;

/**
 * A parameter reader reads an XML file and sets a report's parameter
 * values. This class is used when the report is being run from the
 * command line and the user has given us the name of an XML file containing
 * parameter elements.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class ParameterReader extends MinML {

/**
 * If there is no report element dtd-version attribute, this is the
 * default value to use.
 */
protected static final double DEFAULT_DTD_VERSION = 0.1;

protected Report report;
protected Parameter parameter;
protected String textData;

/**
 * Constructor.
 *
 * @param report the report whose parameters we are setting
 */
public ParameterReader(Report report) {
    this.report = report;
}

/**
 * Reads the named XML file and sets the report's parameters.
 *
 * @param fileName the name of the XML file
 */
public void readFile(String fileName)
    throws FileNotFoundException, IOException, SAXException
{
    parse(new InputStreamReader(new FileInputStream(fileName),
				Report.XML_JAVA_ENCODING));
}

public void startElement(String name, AttributeList attributes)
    throws SAXException
{
    // Get ready to start collecting text
    if (textData == null || textData.length() > 0)
	textData = new String();

    if ("parameter".equals(name)) {
	String id = attributes.getValue("id");
	parameter = report.findParameter(id);
	if (parameter == null)
	    ErrorHandler.error(I18N.get("ParameterReader.unknown_id") + " "
			       + id + I18N.get("ParameterReader.in_xml"));
    }
}

public void endElement(String name) throws SAXException {
    if ("value".equals(name) && parameter != null)
	parameter.addValue(textData);
    else if ("parameter".equals(name))
	parameter = null;
}

/**
 * Reads text data. Text data inside a single tag can be broken up into
 * multiple calls to this method.
 */
public void characters(char ch[], int start, int length) {
    textData += new String(ch, start, length);
}

}
