package jimm.datavision.gui;
import jimm.datavision.*;
import jimm.datavision.gui.cmd.CompoundCommand;
import jimm.datavision.gui.cmd.BoundsCommand;
import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.Iterator;
import javax.swing.*;

/**
 * A field bounds (position and size) editing dialog box.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class BoundsWin extends EditWin implements FieldWidgetWalker {

protected static final int TEXT_FIELD_COLS = 8;

protected FieldWidget fieldWidget;
protected jimm.datavision.Rectangle origBounds;
protected jimm.datavision.Rectangle fieldBounds;
protected JTextField x_text;
protected JTextField y_text;
protected JTextField w_text;
protected JTextField h_text;

/**
 * Constructor; uses format of first selected field.
 *
 * @param designWin the window to which this dialog belongs
 */
public BoundsWin(DesignWin designWin) {
    this(designWin, designWin.firstSelectedField());
}

/**
 * Constructor.
 *
 * @param designWin the window to which this dialog belongs
 * @param fw a field widget from which we will take the format
 */
public BoundsWin(DesignWin designWin, FieldWidget fw) {
    super(designWin, I18N.get("BoundsWin.title"), "BoundsCommand.name");

    fieldWidget = fw;
    origBounds = fieldWidget.getField().getBounds();
    fieldBounds = new jimm.datavision.Rectangle(origBounds);

    buildWindow();
    pack();
    setVisible(true);
}

/**
 * Builds the window contents.
 */
protected void buildWindow() {
    JPanel editorPanel = buildBoundsEditor();

    // OK, Apply, Revert, and Cancel Buttons
    JPanel buttonPanel = closeButtonPanel();

    // Add values and buttons to window
    getContentPane().add(editorPanel, BorderLayout.CENTER);
    getContentPane().add(buttonPanel, BorderLayout.SOUTH);
}

protected JPanel buildBoundsEditor() {
    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());

    x_text = addCoord(panel, I18N.get("BoundsWin.x"), fieldBounds.x,
		      BorderLayout.WEST);
    y_text = addCoord(panel, I18N.get("BoundsWin.y"), fieldBounds.y,
		      BorderLayout.NORTH);
    w_text = addCoord(panel, I18N.get("BoundsWin.width"), fieldBounds.width,
		      BorderLayout.EAST);
    h_text = addCoord(panel, I18N.get("BoundsWin.height"), fieldBounds.height,
		      BorderLayout.SOUTH);

    return panel;
}

protected JTextField addCoord(JPanel parent, String label, double value,
			      String compassPoint)
{
    JPanel coordPanel = new JPanel();
    coordPanel.add(new JLabel(label));
    JTextField text = new JTextField("" + value, TEXT_FIELD_COLS);
    coordPanel.add(text);
    parent.add(coordPanel, compassPoint);
    return text;
}

protected void fillCoords(jimm.datavision.Rectangle bounds) {
    x_text.setText("" + bounds.x);
    y_text.setText("" + bounds.y);
    w_text.setText("" + bounds.width);
    h_text.setText("" + bounds.height);
}

protected void doSave() {
    fieldBounds.setBounds(Double.parseDouble(x_text.getText()),
			  Double.parseDouble(y_text.getText()),
			  Double.parseDouble(w_text.getText()),
			  Double.parseDouble(h_text.getText()));

    if (designWin.countSelectedFields() == 0)
	step(fieldWidget);
    else			// Call step() for all selected fields
	designWin.withSelectedWidgetsDo(this);
}

/**
 * Saves a single field widget.
 *
 * @param fw a field widget
 */
public void step(FieldWidget fw) {
    BoundsCommand command = new BoundsCommand(fw, fieldBounds);
    command.perform();
    commands.add(command);
}

protected void doRevert() {
    fillCoords(origBounds);
}

}
