package jimm.datavision.gui;
import jimm.datavision.ReportReader;
import jimm.datavision.ErrorHandler;
import java.io.*;
import uk.co.wilson.xml.MinML;
import org.xml.sax.*;


/**
 * A database connection reader opens an existing report XML file and
 * reads the database connection information. It is opened when the
 * user clicks "Copy Settings..." from within a database connection
 * window.
 *
 * @see DbConnWin
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class DbConnReader extends ReportReader {

protected String driverClassName;
protected String connInfo;
protected String dbName;
protected String username;

public DbConnReader() {
    super(null);
}

/**
 * This override reads only the database connection information.
 *
 * @param fileName the name of the XML file
 */
public void readFile(String fileName) throws IOException, SAXException {
    parse(new FileReader(fileName));
}

public String getDriverClassName() { return driverClassName; }
public String getConnectionInfo() { return connInfo; }
public String getDbName() { return dbName; }
public String getUserName() { return username; }

/**
 * Reads the database tag and grabs the attributes we want.
 */
public void startElement(String name, AttributeList attributes) {
    if ("database".equals(name)) {
	driverClassName = attributes.getValue("driverClassName");
	connInfo = attributes.getValue("connInfo");
	dbName = attributes.getValue("name");
	username = attributes.getValue("username");
    }
}

public void endElement(String name) {}
public void characters(char ch[], int start, int length) {}

}
