package jimm.datavision.gui;
import jimm.datavision.Report;
import jimm.datavision.Parameter;
import jimm.datavision.Formula;
import jimm.datavision.ErrorHandler;
import jimm.datavision.sql.Column;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.JTextArea;
import javax.swing.text.Document;

/**
 * A text area that accepts drags containing report fields. Used by
 * code editors.
 *
 * @see CodeEditorWin
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class DropListenerTextArea
    extends JTextArea
    implements DropTargetListener
{

protected Report report;

public DropListenerTextArea(Report report, String text) {
    super(text);
    this.report = report;
    new DropTarget(this,	// component
		   DnDConstants.ACTION_COPY_OR_MOVE, // actions
		   this);	// DropTargetListener
}

public void drop(DropTargetDropEvent e) {
    try {
	DataFlavor stringFlavor = DataFlavor.stringFlavor;
	Transferable tr = e.getTransferable();
	if (e.isDataFlavorSupported(stringFlavor)) {
	    String str = (String)tr.getTransferData(stringFlavor);
	    if (str.startsWith("column:")) {
		Column col = report.findColumn(str.substring(7));
		replaceSelection("{" + col.fullName() + "}");
	    }
	    else if (str.startsWith("parameter:")) {
		Parameter param = report.findParameter(str.substring(10));
		replaceSelection(param.designLabel());
	    }
	    else if (str.startsWith("formula:")) {
		Formula formula = report.findFormula(str.substring(8));
		replaceSelection(formula.designLabel());
	    }
	    else
		replaceSelection(str);

	    e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
	    e.dropComplete(true);
	}
	else
	    e.rejectDrop();
    }
    catch(Exception ex) {
	ErrorHandler.error(ex);
    }
}

public void dragEnter(DropTargetDragEvent e) { }
public void dragExit(DropTargetEvent e) { }
public void dragOver(DropTargetDragEvent e) { }
public void dropActionChanged(DropTargetDragEvent e) { }
}
