package jimm.datavision.gui;
import jimm.datavision.*;
import jimm.datavision.sql.Column;
import jimm.datavision.gui.parameter.ParamEditWin;
import javax.swing.JFrame;

/**
 * The classes in this file are leaf nodes for the field picker tree.
 *
 * @see FieldPickerTree
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */

// ================================================================
public abstract class FPLeafInfo {
protected Object leaf;
protected DesignWin designWin;
FPLeafInfo(Draggable obj, DesignWin designWin) {
    leaf = obj;
    this.designWin = designWin;
}
public Object getLeaf() { return leaf; }
public String toString() {
    return (leaf instanceof Nameable)
	? ((Nameable)leaf).getName() : leaf.toString();
}
public String dragString() { return ((Draggable)leaf).dragString(); }
abstract boolean isDeletable();
// Only used for formulas, parameters, and user columns
void openEditor() {}
}

// ================================================================
class ColumnInfo extends FPLeafInfo {
ColumnInfo(Column c, DesignWin designWin) { super(c, designWin); }
boolean isDeletable() { return false; }
}

// ================================================================
class FormulaInfo extends FPLeafInfo {
protected Report report;
FormulaInfo(Report r, Formula f, DesignWin designWin) {
    super(f, designWin);
    report = r;
}
boolean isDeletable() { return !report.containsReferenceTo((Formula)leaf); }
void openEditor() { new FormulaWin(designWin, report, (Formula)leaf); }
}

// ================================================================
class ParameterInfo extends FPLeafInfo {
protected Report report;
ParameterInfo(Report r, Parameter p, DesignWin designWin) {
    super(p, designWin);
    report = r;
}
boolean isDeletable() { return !report.containsReferenceTo((Parameter)leaf); }
void openEditor() { new ParamEditWin(designWin, (Parameter)leaf); }
}

// ================================================================
class UserColumnInfo extends FPLeafInfo {
protected Report report;
UserColumnInfo(Report r, UserColumn uc, DesignWin designWin) {
    super(uc, designWin);
    report = r;
}
boolean isDeletable() { return !report.containsReferenceTo((UserColumn)leaf); }
void openEditor() { new UserColumnWin(designWin, report, (UserColumn)leaf); }
}

// ================================================================
class SpecialInfo extends FPLeafInfo {
protected String specialString;
protected String dragString;
SpecialInfo(String ss, String ds, DesignWin designWin) {
    super(null, designWin);
    specialString = ss;
    dragString = ds;
}
public String toString() { return specialString; }
public String dragString() { return dragString; }
boolean isDeletable() { return false; }
}

//  class SubtotalInfo extends FPLeafInfo {
//  SubtotalInfo(SubtotalField f, DesignWin designWin) { super(f, designWin); }
//  public String toString() { return f.designLabel(); }
//  boolean isDeletable() { return false; }
//  }
