package jimm.datavision.gui;
import jimm.datavision.*;
import jimm.datavision.gui.cmd.FormulaEditCommand;
import java.util.Observable;
import java.util.Observer;

/**
 * This dialog is for editing formula code.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 * @see jimm.datavision.Formula
 * @see FormulaWidget
 * @see jimm.datavision.gui.cmd.FormulaEditCommand
 */
public class FormulaWin extends CodeEditorWin implements Observer {

protected Formula formula;

/**
 * Constructor.
 *
 * @param frame the window to which this dialog belongs
 * @param report the report
 * @param formula the formula whose text needs editing
 */
public FormulaWin(DesignWin designWin, Report report, Formula formula) {
    super(designWin, report, formula.getEditableEvalString(),
	  I18N.get("FormulaWin.title_prefix") + ' ' + formula.getName(),
	  "FormulaWin.error_unchanged", "FormulaWin.error_title");
    this.formula = formula;
    formula.addObserver(this);
}

public void finalize() {
    formula.deleteObserver(this);
}

public void update(Observable o, Object arg) {
    setTitle(I18N.get("FormulaWin.title_prefix") + ' ' + formula.getName());
    codeField.setText(formula.getEditableEvalString());
}

/**
 * Creates and executes a command that changes the formula's eval string.
 * If there is an error, the command is cancelled (never sent to the
 * design window).
 *
 * @param text the new eval string
 */
public void save(String text) {
    formula.deleteObserver(this);
    command = new FormulaEditCommand(formula, text);
}

}
