package jimm.datavision.gui;
import jimm.datavision.*;
import jimm.datavision.sql.Table;
import jimm.datavision.sql.Column;
import jimm.datavision.gui.cmd.GroupEditCommand;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

/**
 * This dialog is used for editing report groups.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class GroupWin extends TwoListWin {

/**
 * Constructor.
 *
 * @param designWin the window to which this dialog belongs
 * @param report the...um...I forgot
 */
public GroupWin(DesignWin designWin, Report report) {
    super(designWin, I18N.get("GroupWin.title"), "GroupChangeCommand.name",
	  "GroupWin.right_box_title", report);
}

protected void fillListModels() {
    // First find and add groups in order.
    for (Iterator iter = report.groups(); iter.hasNext(); ) {
	Group group = (Group)iter.next();
	rightModel.addElement(new GroupWinListItem(group.getColumn(), group));
    }

    // Now iterate through all available columns, adding those not already
    // grouped to the left list.
    Query query = report.getQuery();
    Collection tables = query.getTablesUsed();
    for (Iterator iter = query.getTablesUsed().iterator(); iter.hasNext(); ) {
	Table t = (Table)iter.next();
	for (Iterator cols = t.columns(); cols.hasNext(); ) {
	    Column col = (Column)cols.next();
	    Group group = report.findGroup(col);
	    if (group == null) {
		GroupWinListItem item = new GroupWinListItem(col, group);
		leftModel.add(item);
	    }
	}
    }
}

/**
 * Handles ascending and descending sort order buttons.
 */
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (cmd.equals(I18N.get("GUI.ascending")))
	((GroupWinListItem)rightList.getSelectedValue()).sortOrder =
	    Group.SORT_ASCENDING;
    else if (cmd.equals(I18N.get("GUI.descending")))
	((GroupWinListItem)rightList.getSelectedValue()).sortOrder =
	    Group.SORT_DESCENDING;
    else
	super.actionPerformed(e);
}

protected void doSave() {
    // Turn the model's vector into a collection
    ArrayList items = new ArrayList();
    for (Enumeration enum = rightModel.elements(); enum.hasMoreElements(); )
	items.add(enum.nextElement());

    GroupEditCommand cmd = new GroupEditCommand(report, designWin, items);
    cmd.perform();
    commands.add(cmd);
}

protected void doRevert() {
    // Rebuild list models
    leftModel.removeAllElements();
    rightModel.removeAllElements();
    fillListModels();
    adjustButtons();
}

}
