package jimm.datavision.gui;
import jimm.datavision.I18N;
import jimm.datavision.Report;
import jimm.datavision.Query;
import jimm.datavision.sql.Table;
import jimm.datavision.sql.Column;
import jimm.datavision.gui.cmd.SortEditCommand;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

/**
 * This dialog is used for editing the report query's list of sort
 * orders.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class SortWin extends TwoListWin {

protected Query query;

/**
 * Constructor.
 *
 * @param designWin the window to which this dialog belongs
 * @param report the...um...I forgot
 */
public SortWin(DesignWin designWin, Report report) {
    super(designWin, I18N.get("SortWin.title"), "SortChangeCommand.name",
	  "SortWin.right_box_title", report);
    query = report.getQuery();	// Redundant (see fillListModels) but safe
}

/**
 * Fills the unsorted and sorted columns lists. Excludes colums that are
 * used in report groups.
 */
protected void fillListModels() {
    // Since this method is called from within our superclass ctor, before
    // query gets initialized, we have to grab it here.
    query = report.getQuery();

    Collection tables = query.getTablesUsed();
    for (Iterator iter = query.getTablesUsed().iterator(); iter.hasNext(); ) {
	Table t = (Table)iter.next();
	for (Iterator cols = t.columns(); cols.hasNext(); ) {
	    Column col = (Column)cols.next();
	    if (report.isGroupColumn(col)) // Ignore group columns
		continue;
	    String order = query.sortOrderOf(col);
	    SortWinListItem item = new SortWinListItem(col, order);
	    if (order == null)
		leftModel.add(item);
	    else
		rightModel.addElement(item);
	}
    }
}

/**
 * Handles ascending and descending sort order buttons.
 */
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (cmd.equals(I18N.get("GUI.ascending")))
	((SortWinListItem)rightList.getSelectedValue()).sortOrder = 'a';
    else if (cmd.equals(I18N.get("GUI.descending")))
	((SortWinListItem)rightList.getSelectedValue()).sortOrder = 'd';
    else
	super.actionPerformed(e);
}

protected void doSave() {
    // Turn the model's vector into a collection
    ArrayList items = new ArrayList();
    for (Enumeration enum = rightModel.elements(); enum.hasMoreElements(); )
	items.add(enum.nextElement());

    SortEditCommand cmd = new SortEditCommand(query, items);
    cmd.perform();
    commands.add(cmd);
}

protected void doRevert() {
    // Rebuild list models
    leftModel.removeAllElements();
    rightModel.removeAllElements();
    fillListModels();
    adjustButtons();
}

}
