package jimm.datavision.gui;
import jimm.datavision.*;
import jimm.datavision.gui.cmd.UserColumnEditCommand;
import java.util.Observable;
import java.util.Observer;

/**
 * This dialog is for editing user column code.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 * @see jimm.datavision.UserColumn
 * @see UserColumnWidget
 * @see jimm.datavision.gui.cmd.UserColumnEditCommand
 */
public class UserColumnWin extends CodeEditorWin implements Observer {

protected UserColumn userColumn;

/**
 * Constructor.
 *
 * @param designWin the design window to which this dialog belongs
 * @param report the report
 * @param userColumn the userColumn whose text needs editing
 */
public UserColumnWin(DesignWin designWin, Report report, UserColumn userColumn)
{
    super(designWin, report, userColumn.getEditableUserString(),
	  I18N.get("UserColumnWin.title_prefix") + ' ' + userColumn.getName(),
	  "UserColumnWin.error_unchanged", "UserColumnWin.error_title");
    this.userColumn = userColumn;
    userColumn.addObserver(this);
}

public void finalize() {
    userColumn.deleteObserver(this);
}

public void update(Observable o, Object arg) {
    setTitle(I18N.get("UserColumnWin.title_prefix") + ' '
	     + userColumn.getName());
    codeField.setText(userColumn.getEditableUserString());
}

public void save(String text) {
    userColumn.deleteObserver(this);
    command = new UserColumnEditCommand(userColumn, text);
}

}
