package jimm.datavision.gui.cmd;
import jimm.datavision.I18N;
import jimm.datavision.gui.DesignWin;
import jimm.datavision.gui.FieldWidget;
import jimm.datavision.gui.SectionWidget;
import java.util.*;

/**
 * Deletes a list of field widgets.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class DeleteCommand extends CommandAdapter {

protected DesignWin designWin;
protected Collection fieldWidgets;

/**
 * Constructor.
 */
public DeleteCommand(DesignWin designWin, ArrayList selectedFields) {
    this(designWin, selectedFields, I18N.get("DeleteCommand.name"));
}

/**
 * The delegated constructor.
 */
protected DeleteCommand(DesignWin designWin, ArrayList selectedFields,
			String name)
{
    super(name);
    this.designWin = designWin;
    fieldWidgets = (ArrayList)selectedFields.clone();
}

public void perform() {
    HashSet affectedSections = new HashSet();
    designWin.deselectAll();

    for (Iterator iter = fieldWidgets.iterator(); iter.hasNext(); ) {
	FieldWidget fw = (FieldWidget)iter.next();
	fw.doDelete();		// Widget deletes itself and field from report
	affectedSections.add(fw.getSectionWidget());
    }

    for (Iterator iter = affectedSections.iterator(); iter.hasNext(); )
	((SectionWidget)iter.next()).repaint();

    designWin.enableMenuItems();
}

public void undo() {
    for (Iterator iter = fieldWidgets.iterator(); iter.hasNext(); ) {
	FieldWidget fw = (FieldWidget)iter.next();
	fw.moveToSection(fw.getSectionWidget());
	designWin.select(fw, true, false);
    }
    // Don't need to call DesignWin.enableMenuItems because each call
    // to DesignWin.select already does that.
}

}
