package jimm.datavision.gui.cmd;
import jimm.datavision.I18N;
import jimm.datavision.Report;
import jimm.datavision.Field;
import jimm.datavision.Section;
import jimm.datavision.gui.DesignWin;
import jimm.datavision.gui.FieldWidget;
import jimm.datavision.gui.SectionWidget;

/**
 * A field clipping gets copied to the clipboard when a field widget is
 * cut. It contains not only a field widget but also the widget's original
 * section and section type so pasting
 * <p>
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class FieldClipping implements Pasteable {

protected Field origField;
protected FieldWidget newWidget;
protected SectionWidget origSectionWidget;
protected int sectionArea;
protected SectionResizeCommand sectionResizeCommand;

public FieldClipping(FieldWidget fw) {
    origField = fw.getField();

    // Remember the original section widget so we can attempt to put it
    // there again, even if this field widget gets moved in the mean time.
    origSectionWidget = fw.getSectionWidget();

    // In case the original section no longer exists, or we are pasting
    // into a different report, remember the area of the section so we
    // can paste into the same area later.
    sectionArea = origSectionWidget.getSectionArea();
}

public void paste(DesignWin designWin) {
    Report pasteReport = designWin.getReport();
    Report origReport = origSectionWidget.getReport();
    SectionWidget sw = null;

    if (pasteReport == origReport) {
	if (pasteReport.contains(origSectionWidget.getSection()))
	    sw = origSectionWidget;
	else
	    sw = sectionWidgetBySectionArea(designWin);
    }
    else {			// Different reports; go by section area
	sw = sectionWidgetBySectionArea(designWin);
    }


    Field newField = (Field)origField.clone();
    newWidget = sw.makeWidgetFor(newField);

    sectionResizeCommand = new SectionResizeCommand(sw);
    newWidget.moveToSection(sw); // Possibly resizes section
    sectionResizeCommand.perform();

    designWin.select(newWidget, true, false);

    // Don't need to call DesignWin.enableMenuItems because each call
    // to DesignWin.select already does that.
}

protected SectionWidget sectionWidgetBySectionArea(DesignWin designWin) {
    Report report = designWin.getReport();
    Section s = report.getFirstSectionByArea(sectionArea);
    if (s == null)		// Can be null if area was group, for example
	// Will not be null; there is always at least one report header section
	s = report.getFirstSectionByArea(Report.AREA_REPORT_HEADER);
    return designWin.findSectionWidgetFor(s);
}

public void undo(DesignWin designWin) {
    newWidget.doDelete();	// Widget deletes itself and field from report
    sectionResizeCommand.undo();
    designWin.enableMenuItems();
}

}
