package jimm.datavision.gui.cmd;
import jimm.datavision.I18N;
import jimm.datavision.Field;
import jimm.datavision.Rectangle;
import jimm.datavision.gui.FieldWidget;
import jimm.datavision.gui.SectionWidget;
import jimm.datavision.gui.PreMoveInfo;

/**
 * Moves a single field.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class FieldMoveCommand extends CommandAdapter {

protected FieldWidget fw;
protected Rectangle newBounds;
protected SectionWidget newSectionWidget;
protected SectionResizeCommand sectionResizeCommand;

public FieldMoveCommand(FieldWidget fw, SectionWidget sw) {
    super(I18N.get("FieldMoveCommand.name"));

    this.fw = fw;
    newBounds = new Rectangle(fw.getField().getBounds()); // Make a copy
    newSectionWidget = sw;
    sectionResizeCommand = new SectionResizeCommand(sw);
}

public void perform() {
    fw.getField().getBounds().setBounds(newBounds);
    fw.putDown(newSectionWidget);
    sectionResizeCommand.perform();
}

public void undo() {
    Field f = fw.getField();
    SectionWidget sw = fw.getSectionWidget();
    PreMoveInfo pmi = fw.getPreMoveInfo();
    f.getBounds().setBounds(pmi.bounds); // Move to original bounds
    fw.moveToSection(pmi.sectionWidget); // Move to original section
    sectionResizeCommand.undo();
    if (sw != pmi.sectionWidget)
	sw.repaint();
}

}
