package jimm.datavision.gui.cmd;
import jimm.datavision.I18N;
import jimm.datavision.Query;
import jimm.datavision.sql.Column;
import jimm.datavision.gui.SortWinListItem;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * A command for changing the sort orders in a query.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 * @see jimm.datavision.Query
 * @see jimm.datavision.gui.SortWin
 */
public class SortEditCommand extends CommandAdapter {

protected static final int NO_CHANGE = 0;
protected static final int ONLY_SORTING_CHANGE = 1;
protected static final int DRASTIC_CHANGE = 2;

protected Query query;
protected Collection newSortItems;
protected Collection oldSortItems;

public SortEditCommand(Query query, Collection sortItems) {
    super(I18N.get("SortEditCommand.name"));
    this.query = query;
    this.newSortItems = sortItems;

    // Create list of current sorts
    oldSortItems = new ArrayList();
    for (Iterator iter = query.sortedColumns(); iter.hasNext(); ) {
	Column col = (Column)iter.next();
	oldSortItems.add(new SortWinListItem(col, query.sortOrderOf(col)));
    }
}

public void perform() {
    setSorts(newSortItems);
}

public void undo() {
    setSorts(oldSortItems);
}

protected void setSorts(Collection itemList) {
    query.clearSorts();
    for (Iterator iter = itemList.iterator(); iter.hasNext(); ) {
	SortWinListItem item = (SortWinListItem)iter.next();
	query.addSort(item.getColumn(), item.sortsAscending() ? "asc" : "desc");
    }
}

}
