package jimm.datavision.layout;
import jimm.datavision.*;

/**
 * An XML layout engine.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class XMLLE extends SortedLayoutEngine {

protected IndentWriter iout;
protected String encoding;

/**
 * Constructor.
 *
 * @param out an indent writer
 */
public XMLLE(IndentWriter out) {
    this(out, Report.XML_ENCODING_ATTRIBUTE);
}

/**
 * Constructor. Optionally specify the encoding string to write to the
 * XML file. This string is written as the XMLDecl encoding attribute.
 * Use legal XML values like "UTF-8", not Java values like "UTF8".
 *
 * @param out an indent writer
 * @param enc an XML encoding string; if <code>null</code>, uses
 * <code>Report.XML_ENCODING_ATTRIBUTE</code>
 * @see Report
 */
public XMLLE(IndentWriter out, String enc) {
    super();
    iout = out;
    encoding = (enc == null ? Report.XML_ENCODING_ATTRIBUTE : enc);
}

protected void doStart() {
    iout.println("<?xml version=\"1.0\"? encoding=\"" + encoding + "\"?>");
    iout.println("<!-- Generated by DataVision version " + info.Version
		 + " -->");
    iout.println("<!-- " + info.URL + " -->");
    iout.println("<report>");
    iout.indent();
}

protected void doEnd() {
    iout.outdent();
    iout.println("</report>");
    iout.flush();
}

protected void doStartPage() {
    iout.println("<!-- ============== Page " + pageNumber()
		 + " ============== -->");
    iout.println("<newpage number=\"" + pageNumber() + "\" />");
}

protected void doOutputSection(Section s) {
    iout.println("<section type=\"" + currentSectionTypeAsString() + "\">");
    iout.indent();
    super.doOutputSection(s);
    iout.outdent();
    iout.println("</section>");
}

protected void doOutputField(Field field) {
    iout.print("<field id=\"" + field.getId() + "\" type=\""
	       + field.typeString() + "\"");
    if (field instanceof SpecialField)
	iout.print(" value=\"" + field.getValue() + "\"");
    else if (field instanceof ColumnField)
	iout.print(" column=\"" + ((ColumnField)field).getColumn().fullName()
		   + "\"");
    else if (field instanceof SubtotalField) {
	SubtotalField sf = (SubtotalField)field;
	if (sf.getGroup() != null)
	    iout.print(" group=\"" + sf.getGroup().getColumn().getId() + "\"");
    }

    iout.println("><![CDATA[" + field.toString() + "]]></field>");
}

protected void doOutputImage(ImageField image) {
    doOutputField(image);
}

protected void doOutputLine(Line l) {
    l.writeXML(iout);
}

}

