package jimm.datavision.layout.swing;
import jimm.datavision.*;
import java.awt.Font;
import java.awt.Color;
import java.awt.Dimension;
import javax.swing.JTextPane;
import javax.swing.text.Style;
import javax.swing.text.StyleContext;
import javax.swing.text.StyleConstants;

/**
 * A Swing field is the visual representation of a report field.
 *
 * @see SwingLE
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class SwingTextField extends AbstractSwingField {

protected static final String DEFAULT_FONT_FAMILY = "Serif";

/**
 * Constructor.
 *
 * @param f report field
 */
public SwingTextField(Field f) {
    this(f, f.toString());
}

/**
 * Constructor.
 *
 * @param f report field
 * @param str label text
 */
public SwingTextField(Field f, String str) {
    super(f, new JTextPane());
    JTextPane textPane = (JTextPane)getComponent();
    textPane.setText(str);
    textPane.setEditable(false);
    format();
}

/**
 * Formats the label according to the field's formatting specifications.
 */
public void format() {
    JTextPane textPane = (JTextPane)getComponent();

    Format format = field.getFormat() == null
	? Format.DEFAULT : field.getFormat();
    Style style = StyleContext.getDefaultStyleContext()
	.getStyle(StyleContext.DEFAULT_STYLE);

    // Save selection and select all text
    int selStart = textPane.getSelectionStart();
    int selEnd = textPane.getSelectionEnd();
    textPane.selectAll();

    StyleConstants.setBold(style, format.isBold());
    StyleConstants.setItalic(style, format.isItalic());
    StyleConstants.setUnderline(style, format.isUnderline());
    StyleConstants.setFontSize(style, (int)format.getSize());
    StyleConstants.setFontFamily(style, format.getFont() == null
				 ? DEFAULT_FONT_FAMILY
				 : format.getFont());
    StyleConstants.setForeground(style, format.getColor());

    // Align
    switch (format.getAlign()) {
    case Format.ALIGN_CENTER:
	StyleConstants.setAlignment(style, StyleConstants.ALIGN_CENTER);
	break;
    case Format.ALIGN_RIGHT:
	StyleConstants.setAlignment(style, StyleConstants.ALIGN_RIGHT);
	break;
    default:
	StyleConstants.setAlignment(style, StyleConstants.ALIGN_LEFT);
	break;
    }
    textPane.setParagraphAttributes(style, true);

    // Restore selection
    textPane.setCaretPosition(selStart);
    textPane.moveCaretPosition(selEnd);

    makeBorders();
}

}
