package jimm.datavision.test;
import jimm.datavision.Report;
import jimm.datavision.Formula;
import java.io.*;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;
import org.xml.sax.SAXException;

public class FormulaTest extends TestCase {

protected static final String EXAMPLE_REPORT =
    "jimm/datavision/test/test.xml";

protected Report report;

public static Test suite() {
    return new TestSuite(FormulaTest.class);
}

public FormulaTest(String name) {
    super(name);
}

public void setUp() {
    report = new Report();
}

public void testSingleTokens() throws IOException {
    HashMap testVals = new HashMap();
    Double theAnswer = new Double(42);
    testVals.put("42", theAnswer);
    testVals.put(" 42", theAnswer);
    testVals.put("42 ", theAnswer);
    testVals.put(" 42 ", theAnswer);
    testVals.put(" \t 42 \t", theAnswer);
    testVals.put("66.6", new Double(66.6));
    testVals.put("\"foo\"", "\"foo\"");
    testVals.put("\"don't forget apostrophes\"",
		 "\"don't forget apostrophes\"");
    testVals.put("\"or \\\"quoted quotes\\\" either\"",
		 "\"or \\\"quoted quotes\\\" either\"");

    runTests(testVals);
}

public void testExpressions() throws IOException {
    HashMap testVals = new HashMap();
    Double theAnswer = new Double(42);
    Double aBoringAnswer = new Double(36);

    testVals.put("39 3 +", theAnswer);
    testVals.put(" 39 3 + ", theAnswer);
    testVals.put(" 39\t3 +", theAnswer);
    testVals.put(" 39    3\t+", theAnswer);
    testVals.put("39 3 +", theAnswer);
    testVals.put("45 3 -", theAnswer);
    testVals.put("21 2 *", theAnswer);
    testVals.put("84.00 2.0 /", theAnswer);
    testVals.put(" \"foo\" \"bar\" + ", "\"foobar\"");
    testVals.put("\"foo\" 3 *", "foofoofoo");
    testVals.put("\"foo\" 3.5 *", "foofoofoo");
    testVals.put("42.3343 floor", theAnswer);
    testVals.put("42.3343 round", theAnswer);
    testVals.put("41.3343 ceil", theAnswer);
    testVals.put("6 2 **", aBoringAnswer);
    testVals.put("6 2 pow", aBoringAnswer);
    testVals.put("42 36 max", theAnswer);
    testVals.put("36 42 max", theAnswer);
    testVals.put("42 36 min", aBoringAnswer);
    testVals.put("36 42 min", aBoringAnswer);
    testVals.put("42 abs", theAnswer);
    testVals.put("-42 abs", theAnswer);
    testVals.put("1764 sqrt", theAnswer);
    testVals.put("1764 sqrt 2 ** sqrt", theAnswer);

    runTests(testVals);
}

public void testDisplay()
    throws FileNotFoundException, IOException, SQLException,
	   ClassNotFoundException, InstantiationException,
	   IllegalAccessException, SAXException
{
    report.setDatabasePassword("");
    report.readFile(EXAMPLE_REPORT); // Must come after password set

    Formula f = report.findFormula(new Long(1));
    assertNotNull(f);

    assertEquals("{jobs.hourly_rate} 100.0 /", f.getEditableEvalString());

    f.setEvalString("{?1}");
    assertEquals("{?1}", f.getEvalString());
    assertEquals("{?String Param}", f.getEditableEvalString());

    f.setEditableEvalString("{?String Param}");
    assertEquals("{?1}", f.getEvalString());
    assertEquals("{?String Param}", f.getEditableEvalString());

    f.setEditableEvalString("{!1}");
    assertEquals("{!1}", f.getEvalString());
    assertEquals("{!Short Title}", f.getEditableEvalString());
}

public void runTests(HashMap testVals) throws IOException {
    Formula f = new Formula(null, report, "Unnamed Formula");
    for (Iterator iter = testVals.keySet().iterator(); iter.hasNext(); ) {
	String evalStr = (String)iter.next();
	Object answer = testVals.get(evalStr);

	Object calculated = f.rpnEval(evalStr);
	assertEquals(answer, calculated);
	assertEquals(answer.getClass(), calculated.getClass());
    }
}

public static void main(String[] args) {
    junit.textui.TestRunner.run(suite());
}

}
