package jimm.datavision.test;
import jimm.datavision.Getopts;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

public class GetoptsTest extends TestCase {

public static Test suite() {
    return new TestSuite(GetoptsTest.class);
}

public GetoptsTest(String name) {
    super(name);
}

public void testGetopts() {
    String[] args = {
	"-a", "-b", "-c", "-f", "farg", "-g", "-eearg",
	"-z",			// Illegal arg
	"non-option-arg one", "non-option-arg two",
    };
    Getopts g = new Getopts("abcd:e:f:gh:i", args);

    assertTrue(g.hasOption('a'));
    assertTrue(g.hasOption('b'));
    assertTrue(g.hasOption('c'));
    assertTrue(!g.hasOption('d'));
    assertTrue(g.hasOption('e'));
    assertTrue(g.hasOption('f'));
    assertTrue(g.hasOption('g'));
    assertTrue(!g.hasOption('h'));
    assertTrue(!g.hasOption('i'));

    assertTrue(g.error());	// That -z doesn't belong

    assertEquals("", g.option('d'));
    assertNull(g.option('d', null));
    assertEquals("xyzzy", g.option('d', "xyzzy"));
    assertEquals("earg", g.option('e'));
    assertEquals("farg", g.option('f'));

    assertEquals(2, g.argc());
    assertEquals(args[args.length - 2], g.argv(0));
    assertEquals(args[args.length - 1], g.argv(1));
}

public void testSimpleCommandLine() {
    String[] args = { "-p", "password", "filename" };
    Getopts g = new Getopts("cdhlxnp:s:r:", args);

    assertEquals("password", g.option('p'));
    assertEquals(1, g.argc());
    assertEquals("filename", g.argv(0));
}

public static void main(String[] args) {
    junit.textui.TestRunner.run(suite());
}

}
