package jimm.datavision.test;
import jimm.datavision.*;
import jimm.datavision.sql.Column;
import jimm.datavision.layout.CharSepLE;
import java.io.*;
import java.util.Date;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;
import org.xml.sax.SAXException;

/**
 * Reads a report from an XML file, runs it, and verifies the output. Uses
 * the <code>CharSepLE</code> layout engine to produce a tab-delimited
 * output file.
 * <p>
 * These tests are tightly coupled with the contents of the file
 * <tt>test.xml</tt> and the contents of the test database generated
 * by the files in <tt>jimm/datavision/testdata</tt>.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 * @see jimm.datavision.layout.CharSepLE
 */
public class SubtotalTest extends TestCase {

protected static final String EXAMPLE_REPORT =
    "jimm/datavision/test/subtotal_test.xml";
protected static final String OUT_FILE = "/tmp/datavision_test_out.txt";

protected Report report;

public static Test suite() {
    return new TestSuite(SubtotalTest.class);
}

public SubtotalTest(String name) {
    super(name);
}

public void setUp()
    throws FileNotFoundException, IOException, SQLException,
	   ClassNotFoundException, InstantiationException,
	   IllegalAccessException, SAXException
{
    report = new Report();
    report.setDatabasePassword("");

    File f = new File(OUT_FILE);
    f.deleteOnExit();
    PrintWriter out = new PrintWriter(new FileWriter(f));
    report.setLayoutEngine(new CharSepLE(out, '\t'));

    report.readFile(EXAMPLE_REPORT); // Must come after setting password
}

public void tearDown() {
    new File(OUT_FILE).delete();
}

public void testSubtotal() throws IOException, FileNotFoundException {
    // Run report in this thread, not a separate one. Running the
    // report closes the output stream.
    report.runReport();

    // Open the output and look for various things.
    BufferedReader in = new BufferedReader(new FileReader(OUT_FILE));

    expect("A", in);
    expect("C", in);
    expect(1, in);
    expect(1, in);
    expect(1, in);
    expect(3, in);		// subtotal group 2
    expect(3, in);		// subtotal group 1

    expect("B", in);
    expect("C", in);
    expect(1, in);
    expect(1, in);
    expect(2, in);		// subtotal group 2
    expect(2, in);		// subtotal group 1
    expect(5, in);		// grand total

    // Make sure we are at the end of the file.
    assertNull(in.readLine());

    in.close();
}

protected void expect(String value, BufferedReader in) throws IOException {
    String line = in.readLine();
    assertNotNull(line);
    assertEquals(value, line);
}

protected void expect(int value, BufferedReader in) throws IOException {
    expect("" + value, in);
}

public static void main(String[] args) {
    junit.textui.TestRunner.run(suite());
}

}
