package jimm.datavision.test;
import jimm.datavision.Utils;
import java.util.ArrayList;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

public class UtilsTest extends TestCase {

public static Test suite() {
    return new TestSuite(UtilsTest.class);
}

public UtilsTest(String name) {
    super(name);
}

public void testEscapeXML() {
    assertEquals("", Utils.escapeXML(null));
    assertEquals("abc&amp;def", Utils.escapeXML("abc&def"));
    assertEquals("abc&quot;def&apos;", Utils.escapeXML("abc\"def'"));
    assertEquals("&lt;&gt;", Utils.escapeXML("<>"));
}

public void testUnescapeXML() {
    assertEquals("", Utils.unescapeXML(null));
    assertEquals("abc&def", Utils.unescapeXML("abc&amp;def"));
    assertEquals("abc\"def'", Utils.unescapeXML("abc&quot;def&apos;"));
    assertEquals("<>", Utils.unescapeXML("&lt;&gt;"));
    assertEquals("abc def", Utils.unescapeXML("abc&#20;def"));
}

public void testJoin() {
    assertEquals("", Utils.join(null, null));
    assertEquals("", Utils.join(null, ","));

    ArrayList list = new ArrayList();
    assertEquals("", Utils.join(list, null));
    assertEquals("", Utils.join(list, ","));

    list.add("a");
    assertEquals("a", Utils.join(list, null));
    assertEquals("a", Utils.join(list, ","));

    list.add("xyzzy");
    assertEquals("axyzzy", Utils.join(list, null));
    assertEquals("a,xyzzy", Utils.join(list, ","));
    assertEquals("a-*-xyzzy", Utils.join(list, "-*-"));
}

public static void main(String[] args) {
    junit.textui.TestRunner.run(suite());
}

}
